package fr.soleil.docking.infonode;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import net.infonode.docking.RootWindow;
import net.infonode.docking.View;
import net.infonode.docking.ViewSerializer;
import net.infonode.docking.properties.DockingWindowProperties;
import net.infonode.docking.util.DockingUtil;
import net.infonode.util.Direction;
import fr.soleil.docking.ADockingManager;
import fr.soleil.docking.exception.DockingException;
import fr.soleil.docking.infonode.view.InfoNodeViewFactory;
import fr.soleil.docking.perspective.IPerspective;
import fr.soleil.docking.perspective.IPerspectiveFactory;
import fr.soleil.docking.perspective.PerspectiveFactory;
import fr.soleil.docking.view.IView;
import fr.soleil.docking.view.IViewFactory;

/**
 * A class that prepares docking. Obtain your RootWindow from this class.
 * 
 * @author HARDION
 * @author GIRARDOT
 */
public class InfoNodeDockingManager extends ADockingManager {

    protected RootWindow rootWindow;

    public InfoNodeDockingManager() {
        this(new InfoNodeViewFactory(), new PerspectiveFactory());
    }

    protected InfoNodeDockingManager(IViewFactory viewFactory, IPerspectiveFactory perspectiveFactory) {
        super(viewFactory, perspectiveFactory);
    }

    @Override
    public JComponent getDockingArea() {
        if (rootWindow == null) {
            rootWindow = (RootWindow) createNewDockingArea(null);
        }
        return rootWindow;
    }

    @Override
    public JComponent createNewDockingArea(Color background) {
        RootWindow result = new RootWindow(new MyViewSerializer(viewFactory));
        if (background != null) {
            result.getRootWindowProperties().getWindowAreaProperties().setBackgroundColor(background);
        }
        for (IView view : viewFactory.getViews()) {
            DockingUtil.addWindow((View) view, rootWindow);
        }
        result.getWindowBar(Direction.DOWN).setEnabled(true);
        result.getRootWindowProperties().getDockingWindowProperties().setUndockEnabled(false);
        return result;
    }

    @Override
    public void setDockingAreaBeackground(JComponent dockingArea, Color areaBackground) {
        if (dockingArea instanceof RootWindow) {
            RootWindow rootWindow = (RootWindow) dockingArea;
            rootWindow.getRootWindowProperties().getWindowAreaProperties().setBackgroundColor(areaBackground);
        }
    }

    @Override
    public void loadPerspective(IPerspective perspective) throws DockingException {
        DockingException dockingException = null;
        if (perspective.getByteArray().length > 0) {
            ObjectInputStream ois = null;
            ByteArrayInputStream bais = null;
            try {
                bais = new ByteArrayInputStream(perspective.getByteArray());
                ois = new ObjectInputStream(bais);
                rootWindow.read(ois);
            } catch (Exception e) {
                dockingException = new DockingException("Perspective.load() :" + " Unexpected Error", e);
            } finally {
                try {
                    if (ois != null) {
                        ois.close();
                    }
                    // Actually there is no effect to close bais ...
                } catch (IOException e) {
                    if (dockingException == null) {
                        dockingException = new DockingException("I/O Exception", e);
                    } else {
                        e.printStackTrace();
                    }
                }
            }
        }
        if (dockingException != null) {
            throw dockingException;
        }
    }

    @Override
    protected void savePerspective(IPerspective perspective) {
        ByteArrayOutputStream baos = null;
        ObjectOutputStream ous = null;
        baos = new ByteArrayOutputStream();
        try {
            ous = new ObjectOutputStream(baos);
            rootWindow.write(ous, false);
        } catch (IOException e) {
            e.printStackTrace();
            JOptionPane.showMessageDialog(rootWindow, "Perspective.save() :" + " Unexpected Error (see traces)",
                    "INFONODE Docking - Error", JOptionPane.ERROR_MESSAGE);
        } finally {
            try {
                ous.close();
                // Actually there is no effect to close baos ...
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        perspective.setByteArray(baos.toByteArray());
    }

    @Override
    protected void updateViews(PropertyChangeEvent evt) {
        if (evt.getSource() == this.viewFactory) {
            View newView = (View) evt.getNewValue();
            View oldView = (View) evt.getOldValue();
            if (newView == null) { // removed
                oldView.close();
            } else {
                DockingUtil.addWindow(newView, rootWindow);
            }
        }
    }

    @Override
    public void setEnabledDocking(boolean enabledDocking) {
        RootWindow rootWindow = (RootWindow) getDockingArea();
        rootWindow.getRootWindowProperties().getDockingWindowProperties().setCloseEnabled(enabledDocking);
        rootWindow.getRootWindowProperties().getDockingWindowProperties().setDragEnabled(enabledDocking);
        rootWindow.getRootWindowProperties().getDockingWindowProperties().setMaximizeEnabled(enabledDocking);
        rootWindow.getRootWindowProperties().getDockingWindowProperties().setMinimizeEnabled(enabledDocking);
    }

    @Override
    public boolean isEnabledDocking() {
        DockingWindowProperties dwp = this.rootWindow.getRootWindowProperties().getDockingWindowProperties();

        return dwp.getCloseEnabled() && dwp.getDragEnabled() && dwp.getMaximizeEnabled() && dwp.getMinimizeEnabled();
    }

    // ///////////// //
    // Inner classes //
    // ///////////// //

    protected class MyViewSerializer implements ViewSerializer {

        private final IViewFactory viewFactory;

        /**
         * Utility constructor that creates a map with a number of views. A view gets it's index in
         * the array as id.
         * 
         * @param views the views to add to the map
         */
        public MyViewSerializer(IViewFactory viewFactory) {
            this.viewFactory = viewFactory;
        }

        @Override
        public View readView(ObjectInputStream in) throws IOException {
            Object id = viewFactory.readViewId(in);
            return (View) viewFactory.getView(id);
        }

        @Override
        public void writeView(View view, ObjectOutputStream out) throws IOException {
            viewFactory.writeViewId(((IView) view).getId(), out);
        }

    }

}
